package run.evan.gost;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.ColorStateList;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.text.InputType;
import android.text.method.ScrollingMovementMethod;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.ref.WeakReference;
import java.util.Map;

import run.evan.gost.databinding.ActivityMainBinding;

public class MainActivity extends Activity {

    private ActivityMainBinding binding;

    Process process;

    SharedPreferences configPreferences;

    SharedPreferences appPreferences;

    ArrayAdapter<String> starAdapter;

    MyHandler handler = new MyHandler(this);

    Intent gostServiceIntent;

    private static final String TAG = "run.evan.gost";


    private static class MyHandler extends Handler {


        private WeakReference<MainActivity> weakReference;

        public MyHandler(MainActivity activity) {
            this.weakReference = new WeakReference(activity);
        }

        @Override
        public void handleMessage(Message msg) {
            MainActivity activity = weakReference.get();
            super.handleMessage(msg);


            if (null != activity) {
                switch (msg.what) {
                    case 1:
                        int length = activity.binding.textView.getText().length();
                        String s = activity.binding.textView.getText().toString();
                        if (length > 5000) {
                            activity.binding.textView.setText(s.substring(2500));
                        }
                        activity.binding.textView.append(msg.obj.toString());
                        activity.binding.textView.setSelection(activity.binding.textView.getText().length());
                        break;
                    case 2:
                        activity.binding.btnStart.setText(R.string.btn_stop);
                        activity.binding.btnStart.setBackgroundTintList(ColorStateList.valueOf(-1499549));
                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                            activity.getApplicationContext().startForegroundService(activity.gostServiceIntent);
                        } else {
                            activity.getApplicationContext().startService(activity.gostServiceIntent);
                        }
                        break;
                    case 3:
                        activity.binding.btnStart.setText(R.string.btn_start);
                        activity.binding.btnStart.setBackgroundTintList(ColorStateList.valueOf(-11751600));
                        activity.getApplicationContext().stopService(activity.gostServiceIntent);
                    default:
                }
            }
        }
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR);
        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        gostServiceIntent = new Intent(MainActivity.this, GostService.class);
        appPreferences = getSharedPreferences("app", MODE_PRIVATE);

        if (checkSelfPermission(Manifest.permission.READ_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED
                || checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED
                || checkSelfPermission(Manifest.permission.MOUNT_UNMOUNT_FILESYSTEMS) != PackageManager.PERMISSION_GRANTED
        ) {
            requestPermissions(new String[]{Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.MOUNT_UNMOUNT_FILESYSTEMS}, 10);
        }
        // android 11  且 不是已经被拒绝
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R && !appPreferences.getBoolean("refuse", false)) {
            // 先判断有没有权限
            if (!Environment.isExternalStorageManager()) {
                Intent intent = new Intent(Settings.ACTION_MANAGE_APP_ALL_FILES_ACCESS_PERMISSION);
                intent.setData(Uri.parse("package:" + getPackageName()));
                startActivityForResult(intent, 1024);
            }
        }

        binding.textView.setMovementMethod(ScrollingMovementMethod.getInstance());
        configPreferences = getSharedPreferences("config", MODE_PRIVATE);


        starAdapter = new ArrayAdapter<String>(this, R.layout.item_select);
        starAdapter.setDropDownViewResource(R.layout.item_dropdown);
        binding.spinner.setAdapter(starAdapter);
        binding.configEditText.setVisibility(View.GONE);
        reloadConfig();
        //给下拉框设置选择监听器，一旦用户选中某一项，就触发监听器的onItemSelected方法
        binding.spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
                binding.configEditText.setText(configPreferences.getString(adapterView.getSelectedItem().toString(), ""));
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {
                binding.configEditText.setText("");
            }
        });


        MainActivity mainActivity = this;
        binding.btnSave.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                int visibility = binding.configEditText.getVisibility();
                if (visibility == View.GONE) {
                    binding.configEditText.setVisibility(View.VISIBLE);
                    binding.btnSave.setText(R.string.btn_save);
                    return;
                }

                String config = binding.configEditText.getText().toString();
                if ("".contains(config)) {
                    Toast.makeText(getApplicationContext(), R.string.toast_tip_config_empty, Toast.LENGTH_LONG).show();
                    return;
                }

                AlertDialog.Builder builder = new AlertDialog.Builder(mainActivity);
                builder.setTitle(R.string.title_config_box);
                final EditText input = new EditText(mainActivity);
                input.setInputType(InputType.TYPE_CLASS_TEXT);
                Object selectedItem = binding.spinner.getSelectedItem();
                input.setText(selectedItem == null ? "" : selectedItem.toString());
                builder.setView(input);

                builder.setPositiveButton(R.string.title_btn_save, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        String configName = input.getText().toString();
                        if ("".contains(configName)) {
                            Toast.makeText(getApplicationContext(), R.string.toast_tip_empty, Toast.LENGTH_LONG).show();
                            return;
                        }
                        SharedPreferences.Editor edit = configPreferences.edit();
                        edit.putString(configName, config);
                        edit.apply();
                        reloadConfig();
                        binding.btnSave.setText(R.string.btn_edit);
                        binding.configEditText.setVisibility(View.GONE);
                    }
                });
                builder.setNegativeButton(R.string.title_btn_cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.cancel();
                        binding.btnSave.setText(R.string.btn_edit);
                        binding.configEditText.setVisibility(View.GONE);
                    }
                });

                builder.show();


            }
        });

        binding.btnDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                if (null == binding.spinner.getSelectedItem()) {
                    Toast.makeText(getApplicationContext(), R.string.tip_config_not_selected, Toast.LENGTH_LONG).show();
                    return;
                }

                AlertDialog.Builder builder = new AlertDialog.Builder(mainActivity);
                builder.setTitle(R.string.title_confirm);
                builder.setPositiveButton(R.string.title_yes, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        String key = binding.spinner.getSelectedItem().toString();
                        SharedPreferences.Editor edit = configPreferences.edit();
                        edit.remove(key);
                        edit.apply();
                        reloadConfig();
                    }
                });
                builder.setNegativeButton(R.string.title_no, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.cancel();
                    }
                });

                builder.show();
            }
        });

        binding.btnStart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                String basedir = getApplication().getApplicationInfo().nativeLibraryDir;
                String config = binding.configEditText.getText().toString();

                if (null != binding.spinner.getSelectedItem()) {
                    String key = binding.spinner.getSelectedItem().toString();
                    appPreferences.edit().putString("lastConfig", key).apply();
                }


                if (process == null) {
                    if ("".contains(config)) {
                        Toast.makeText(getApplicationContext(), R.string.toast_tip_config_empty, Toast.LENGTH_LONG).show();
                        return;
                    }
                    new Thread(new Runnable() {
                        @Override
                        public void run() {

                            try {

                                process = Runtime.getRuntime().exec(basedir + "/libgost.so " + config);
                                BufferedReader successResult = new BufferedReader(new InputStreamReader(process.getInputStream()));
                                BufferedReader errorResult = new BufferedReader(new InputStreamReader(process.getErrorStream()));


                                new Thread(new Runnable() {
                                    @Override
                                    public void run() {
                                        try {
                                            String s;
                                            while ((s = successResult.readLine()) != null) {
                                                if (s.startsWith("WARNING: linker:")) {
                                                    continue;
                                                }
                                                sendTextToUI(s + "\n");
                                            }
                                        } catch (Exception e) {

                                        }
                                    }
                                }).start();

                                new Thread(new Runnable() {
                                    @Override
                                    public void run() {
                                        try {
                                            String s;
                                            while ((s = errorResult.readLine()) != null) {
                                                if (s.startsWith("WARNING: linker:")) {
                                                    continue;
                                                }
                                                sendTextToUI(s + "\n");
                                            }
                                        } catch (Exception e) {

                                        }
                                    }
                                }).start();

                                Message msg = new Message();
                                msg.what = 2;
                                handler.sendMessage(msg);

                                process.waitFor();
                                process.destroy();
                                process = null;
                                Message msg2 = new Message();
                                msg2.what = 3;
                                handler.sendMessage(msg2);

                            } catch (Exception e) {
                                sendTextToUI(e.getMessage());
                                Message msg = new Message();
                                msg.what = 3;
                                handler.sendMessage(msg);
                            }
                        }
                    }).start();

                } else {
                    process.destroy();
                }


            }
        });

    }


    // 带回授权结果
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 1024 && Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            // 检查是否有权限
            if (!Environment.isExternalStorageManager()) {
                appPreferences.edit().putBoolean("refuse", true).apply();
            }
        }
    }

    private void reloadConfig() {
        starAdapter.clear();
        Map<String, ?> all = configPreferences.getAll();
        String lastConfig = appPreferences.getString("lastConfig", "");
        int count = 0;
        boolean lastValid = false;
        for (String s : all.keySet()) {
            starAdapter.add(s);
            if (s.equals(lastConfig)) {
                binding.spinner.setSelection(count);
                lastValid = true;
            }
            count++;
        }
        if (!lastValid) {
            appPreferences.edit().putString("lastConfig", "").apply();
        }
    }

    private void sendTextToUI(String str) {
        Message msg = new Message();
        msg.what = 1;
        msg.obj = str;
        handler.sendMessage(msg);
    }

}